@@ -51,6 +51,8 @@ DataStore::DataStore(
   username(username),
   password(password),
   isReauthing(false),
+  changingPlayerState(false),
+  clearingCurrentSong(false),
   currentSongId(-1)
 {
   serverConnection = new UDJServerConnection(this);
@@ -62,26 +64,26 @@ DataStore::DataStore(
   }
   activePlaylistRefreshTimer = new QTimer(this);
   activePlaylistRefreshTimer->setInterval(5000);
+  participantRefreshTimer = new QTimer(this);
+  participantRefreshTimer->setInterval(5000);
   setupDB();
 
-  connect(
-    serverConnection,
-    SIGNAL(playerPasswordSet(const QString&)),
-    this,
-    SLOT(onPlayerPasswordSet(const QString&)));
+  connect(serverConnection,
+      SIGNAL(playerStateSet(const QString&)),
+      this,
+      SLOT(onPlayerStateSet(const QString&)));
 
   connect(
     serverConnection,
-    SIGNAL(playerPasswordSetError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)),
+    SIGNAL(playerStateSetError(const QString&, const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)),
     this,
-    SLOT(onPlayerPasswordSetError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)));
-
+    SLOT(onPlayerStateSetError(const QString&, const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)));
 
   connect(
     serverConnection,
-    SIGNAL(playerLocationSet(const QString&, const QString&, const QString&, int)),
+    SIGNAL(playerPasswordSetError(const QString&, const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)),
     this,
-    SLOT(onPlayerLocationSet(const QString&, const QString&, const QString&, int)));
+    SLOT(onPlayerPasswordSetError(const QString&, const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)));
 
   connect(
     serverConnection,
@@ -89,30 +91,12 @@ DataStore::DataStore(
     this,
     SLOT(onPlayerLocationSetError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)));
 
-  connect(
-    serverConnection,
-    SIGNAL(playerPasswordRemoved()),
-    this,
-    SLOT(onPlayerPasswordRemoved()));
-
   connect(
     serverConnection,
     SIGNAL(playerPasswordRemoveError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)),
     this,
     SLOT(onPlayerPasswordRemoveError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)));
 
-  connect(
-    serverConnection,
-    SIGNAL(playerNameChanged(const QString&)),
-    this,
-    SLOT(onPlayerNameChanged(const QString&)));
-
-  connect(
-    serverConnection,
-    SIGNAL(playerNameChangeError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)),
-    this,
-    SLOT(onPlayerNameChangeError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)));
-
   connect(
     serverConnection,
     SIGNAL(libSongsSyncedToServer(const QSet<library_song_id_t>&)),
@@ -155,10 +139,10 @@ DataStore::DataStore(
     SLOT(refreshActivePlaylist()));
 
   connect(
-    serverConnection,
-    SIGNAL(playerStateSet(const QString&)),
+    participantRefreshTimer,
+    SIGNAL(timeout()),
     this,
-    SLOT(onPlayerStateChanged(const QString&)));
+    SLOT(refreshParticipantList()));
 
   connect(
     serverConnection,
@@ -203,6 +187,25 @@ DataStore::DataStore(
     this,
     SLOT(onAuthFail(const QString&)));
 
+  connect(
+    serverConnection,
+    SIGNAL(currentSongCleared()),
+    this,
+    SLOT(onCurrentSongCleared()));
+
+  connect(
+    serverConnection,
+    SIGNAL(currentSongClearError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)),
+    this,
+    SLOT(onCurrentSongClearError(const QString&, int, const QList<QNetworkReply::RawHeaderPair>&)));
+
+
+  connect(
+    serverConnection,
+    SIGNAL(newParticipantList(const QVariantList&)),
+    this,
+    SLOT(onNewParticipantList(const QVariantList&)));
+
 }
 
 void DataStore::setupDB(){
@@ -243,59 +246,134 @@ void DataStore::setupDB(){
 
 }
 
-void DataStore::removePlayerPassword(){
-  serverConnection->removePlayerPassword();
+void DataStore::startPlaylistAutoRefresh(){
+  Logger::instance()->log("Starting playlist auto refresh");
+  activePlaylistRefreshTimer->start();
 }
 
-void DataStore::onPlayerPasswordRemoved(){
+void DataStore::startParticipantsAutoRefresh(){
+  Logger::instance()->log("Starting particpants auto refresh");
+  participantRefreshTimer->start();
+}
+
+void DataStore::clearCurrentSong(){
+  currentSongId = -1;
+  clearingCurrentSong = true;
+  serverConnection->clearCurrentSong();
+}
+
+void DataStore::onCurrentSongCleared(){
+  clearingCurrentSong = false;
+}
+
+void DataStore::onCurrentSongClearError(
+  const QString& errMessage,
+  int errorCode,
+  const QList<QNetworkReply::RawHeaderPair>& headers)
+{
+  if(isTicketAuthError(errorCode, headers)){
+    Logger::instance()->log("Got the ticket-hash challenge");
+    reauthActions.insert(CLEAR_CURRENT_SONG);
+    initReauth();
+  }
+  else{
+    clearingCurrentSong = false;
+    emit clearCurrentSongError(errMessage);
+  }
+
+}
+
+
+void DataStore::onPlayerStateSet(const QString& state){
+  changingPlayerState = false;
+  if(state == getInactiveState()){
+    emit playerSuccessfullySetInactive();
+  }
+}
+
+void DataStore::onPlayerStateSetError(
+    const QString& state,
+    const QString& errMessage,
+    int errorCode,
+    const QList<QNetworkReply::RawHeaderPair>& headers)
+{
+  if(isTicketAuthError(errorCode, headers)){
+    Logger::instance()->log("Got the ticket-hash challenge");
+    if(state != getInactiveState()){
+      reauthActions.insert(SET_PLAYER_STATE);
+    }
+    else{
+      reauthActions.insert(SET_PLAYER_INACTIVE);
+    }
+    initReauth();
+  }
+  else{
+    changingPlayerState = false;
+    if(state == getPlayingState()){
+      emit playPlayerError(errMessage);
+    }
+    else if(state == getPausedState()){
+      emit pausePlayerError(errMessage);
+    }
+    else if(state == getInactiveState()){
+      emit playerSetInactiveError(errMessage);
+    }
+  }
+}
+
+
+void DataStore::removePlayerPassword(){
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-  settings.remove(getPlayerPasswordSettingName());
+  settings.setValue(getHasPlayerPasswordSettingName(), false);
+  serverConnection->removePlayerPassword();
   emit playerPasswordRemoved();
 }
 
 void DataStore::onPlayerPasswordRemoveError(
   const QString& errMessage,
-  int /*errorCode*/,
-  const QList<QNetworkReply::RawHeaderPair>& /*headers*/)
+  int errorCode,
+  const QList<QNetworkReply::RawHeaderPair>& headers)
 {
-  //TODO handle reauth error
-  emit playerPasswordRemoveError(errMessage);
+  if(isTicketAuthError(errorCode, headers)){
+    Logger::instance()->log("Got the ticket-hash challenge");
+    reauthActions.insert(REMOVE_PLAYER_PASSWORD);
+    initReauth();
+  }
+  else{
+    emit playerPasswordRemoveError(errMessage);
+  }
 }
 
 
 void DataStore::setPlayerPassword(const QString& newPassword){
-  serverConnection->setPlayerPassword(newPassword);
-}
-
-void DataStore::onPlayerPasswordSet(const QString& password){
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-  settings.setValue(getPlayerPasswordSettingName(), password);
+  settings.setValue(getHasPlayerPasswordSettingName(), true);
+  serverConnection->setPlayerPassword(newPassword);
   emit playerPasswordSet();
 }
 
 void DataStore::onPlayerPasswordSetError(
+  const QString& attemptedPassword,
   const QString& errMessage,
-  int /*errorCode*/,
-  const QList<QNetworkReply::RawHeaderPair>& /*headers*/)
+  int errorCode,
+  const QList<QNetworkReply::RawHeaderPair>& headers)
 {
-  emit playerPasswordSetError(errMessage);
+  if(isTicketAuthError(errorCode, headers)){
+    Logger::instance()->log("Got the ticket-hash challenge");
+    reauthActions.insert(SET_PLAYER_PASSWORD);
+    reauthPlayerPassword = attemptedPassword;
+    initReauth();
+  }
+  else{
+    emit playerPasswordSetError(errMessage);
+  }
 }
 
 void DataStore::setPlayerLocation(
   const QString& streetAddress,
   const QString& city,
   const QString& state,
-  int zipcode
-)
-{
-  serverConnection->setPlayerLocation(streetAddress, city, state, zipcode);
-}
-
-void DataStore::onPlayerLocationSet(
-  const QString& streetAddress,
-  const QString& city,
-  const QString& state,
-  int zipcode
+  const QString& zipcode
 )
 {
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
@@ -303,46 +381,27 @@ void DataStore::onPlayerLocationSet(
   settings.setValue(getCitySettingName(), city);
   settings.setValue(getStateSettingName(), state);
   settings.setValue(getZipCodeSettingName(), zipcode);
+  serverConnection->setPlayerLocation(streetAddress, city, state, zipcode);
   emit playerLocationSet();
 }
 
+
 void DataStore::onPlayerLocationSetError(
   const QString& errMessage,
-  int /*errorCode*/,
-  const QList<QNetworkReply::RawHeaderPair>& /*headers*/)
-{
-  //TODO handle reauth error
-  emit playerLocationSetError(errMessage);
-}
-
-void DataStore::setPlayerName(const QString& newName){
-  serverConnection->setPlayerName(newName);
-}
-
-void DataStore::onPlayerNameChanged(const QString& newName){
-  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-  settings.setValue(getPlayerNameSettingName(), newName);
-  emit playerNameChanged(newName);
-}
-
-void DataStore::onPlayerNameChangeError(
-  const QString& /*errMessage*/,
   int errorCode,
-  const QList<QNetworkReply::RawHeaderPair>& /*headers*/)
+  const QList<QNetworkReply::RawHeaderPair>& headers)
 {
-  //TODO if reauth error, should reauth
-  if(errorCode == 409){
-    emit playerNameChangeError(tr(
-      "You already have a player with that name."
-    ));
+  if(isTicketAuthError(errorCode, headers)){
+    Logger::instance()->log("Got the ticket-hash challenge");
+    reauthActions.insert(SET_PLAYER_LOCATION);
+    initReauth();
   }
   else{
-  emit playerNameChangeError(tr(
-    "We seem to be having some techincal difficulties and couldn't change "
-    "the name of your player. Try again in a little bit."
-  ));
+    //TODO handle location not found error
+    emit playerLocationSetError(errMessage);
   }
 }
+
 void DataStore::pausePlayer(){
   setPlayerState(getPausedState());
 }
@@ -352,9 +411,16 @@ void DataStore::playPlayer(){
 }
 
 void DataStore::setPlayerState(const QString& newState){
+  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
+  settings.setValue(getPlayerStateSettingName(), newState);
+  changingPlayerState = true;
   serverConnection->setPlayerState(newState);
 }
 
+void DataStore::setPlayerInactive(){
+  serverConnection->setPlayerState(getInactiveState());
+}
+
 
 void DataStore::addMusicToLibrary(
   const QList<Phonon::MediaSource>& songs, QProgressDialog* progress)
@@ -452,6 +518,23 @@ void DataStore::addSongToLibrary(const Phonon::MediaSource& song, QSqlQuery &add
     library_song_id_t)
 }
 
+bool DataStore::alreadyHaveSongInLibrary(const QString& fileName) const{
+  QSqlQuery existsQuery(database);
+  existsQuery.prepare(
+    "SELECT * FROM "+getLibraryTableName()+ " WHERE " + 
+    getLibIsDeletedColName() + "=0 and " + 
+    getLibFileColName() + "=\""+fileName+"\";"
+  );
+
+  EXEC_SQL(
+    "Error executing already in library test query",
+    existsQuery.exec(),
+    existsQuery)
+
+  return existsQuery.next();
+
+}
+
 void DataStore::removeSongsFromLibrary(const QSet<library_song_id_t>& toRemove,
   QProgressDialog* progress)
 {
@@ -530,6 +613,7 @@ DataStore::song_info_t DataStore::takeNextSongToPlay(){
     "SELECT " + getLibFileColName() + ", " + 
     getLibSongColName() + ", " +
     getLibArtistColName() + ", " +
+    getLibDurationColName() + ", " +
     getActivePlaylistLibIdColName() +" FROM " +
     getActivePlaylistViewName() + " LIMIT 1;", 
     database);
@@ -539,12 +623,11 @@ DataStore::song_info_t DataStore::takeNextSongToPlay(){
     nextSongQuery)
   nextSongQuery.next();
   if(!nextSongQuery.isValid()){
-    currentSongId = -1;
-    song_info_t toReturn = {Phonon::MediaSource(""), "", "" };
+    song_info_t toReturn = {Phonon::MediaSource(""), "", "", "" };
     return toReturn;
   }
   currentSongId =
-    nextSongQuery.value(3).value<library_song_id_t>();
+    nextSongQuery.value(4).value<library_song_id_t>();
 
   deleteSongFromPlaylist(currentSongId);
 
@@ -552,10 +635,12 @@ DataStore::song_info_t DataStore::takeNextSongToPlay(){
   serverConnection->setCurrentSong(currentSongId);
 
   QString filePath = nextSongQuery.value(0).toString();
+  QTime qtime(0, nextSongQuery.value(3).toInt()/60, nextSongQuery.value(3).toInt()%60);
   song_info_t toReturn = {
     Phonon::MediaSource(filePath),
     nextSongQuery.value(1).toString(),
-    nextSongQuery.value(2).toString()
+    nextSongQuery.value(2).toString(),
+    qtime.toString("mm:ss")
   };
 
   return toReturn;
@@ -578,7 +663,8 @@ void DataStore::setCurrentSong(const library_song_id_t& songToPlay){
   QSqlQuery getSongQuery(
     "SELECT " + getLibFileColName() + ", " +
     getLibSongColName() + ", " +
-    getLibArtistColName() + " FROM " +
+    getLibArtistColName() + ", " +
+    getLibDurationColName() + " FROM " +
     getActivePlaylistViewName() + " WHERE " + 
     getActivePlaylistLibIdColName() + " = " + QString::number(songToPlay) + ";", 
     database);
@@ -593,10 +679,12 @@ void DataStore::setCurrentSong(const library_song_id_t& songToPlay){
     currentSongId = songToPlay;
     serverConnection->setCurrentSong(songToPlay);
     Logger::instance()->log("Retrieved Artist " + getSongQuery.value(2).toString());
+    QTime qtime(0, getSongQuery.value(3).toInt()/60, getSongQuery.value(3).toInt()%60);
     song_info_t toEmit = {
       Phonon::MediaSource(filePath),
       getSongQuery.value(1).toString(),
-      getSongQuery.value(2).toString()
+      getSongQuery.value(2).toString(),
+      qtime.toString("mm:ss")
     };
     emit manualSongChange(toEmit);
   }
@@ -608,7 +696,7 @@ void DataStore::createNewPlayer(
 {
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
   settings.setValue(getPlayerNameSettingName(), name);
-  settings.setValue(getPlayerPasswordSettingName(), password);
+  settings.setValue(getHasPlayerPasswordSettingName(), true);
   serverConnection->createPlayer(name, password);
 }
 
@@ -618,11 +706,11 @@ void DataStore::createNewPlayer(
   const QString& streetAddress,
   const QString& city,
   const QString& state,
-  const int& zipcode)
+  const QString& zipcode)
 {
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
   settings.setValue(getPlayerNameSettingName(), name);
-  settings.setValue(getPlayerPasswordSettingName(), password);
+  settings.setValue(getHasPlayerPasswordSettingName(), true);
   settings.setValue(getAddressSettingName(), streetAddress);
   settings.setValue(getCitySettingName(), city);
   settings.setValue(getStateSettingName(), state);
@@ -636,7 +724,7 @@ void DataStore::createNewPlayer(
     zipcode);
 }
 
-void DataStore::changeVolumeSilently(qreal newVolume){
+void DataStore::setVolume(qreal newVolume){
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
   Logger::instance()->log("Current volume " + QString::number(settings.value(getPlayerVolumeSettingName()).toReal()));
   Logger::instance()->log("New volume " + QString::number(newVolume));
@@ -668,7 +756,7 @@ void DataStore::syncLibrary(){
   while(needAddSongs.next()){
     currentRecord = needAddSongs.record();
     QVariantMap songToAdd;
-    songToAdd["id"] = currentRecord.value(getLibIdColName());
+    songToAdd["id"] = currentRecord.value(getLibIdColName()).toString();
     QString title = currentRecord.value(getLibSongColName()).toString();
     title.truncate(199);
     songToAdd["title"] = title;
@@ -698,7 +786,7 @@ void DataStore::syncLibrary(){
   QVariantList songsToDelete;
   while(needDeleteSongs.next()){
     currentRecord = needDeleteSongs.record();
-    songsToDelete.append(currentRecord.value(getLibIdColName()));
+    songsToDelete.append(currentRecord.value(getLibIdColName()).toString());
   }
 
   Logger::instance()->log("Found " + QString::number(songsToDelete.size()) + " songs which need deleting");
@@ -824,15 +912,22 @@ void DataStore::setActivePlaylist(const QVariantMap& newPlaylist){
     emit volumeChanged(retrievedVolume/10.0);
   }
 
-  onPlayerStateChanged(newPlaylist["state"].toString());
+  QString retrievedState = newPlaylist["state"].toString();
+  if(!changingPlayerState && retrievedState != getPlayerState()){
+    QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
+    settings.setValue(getPlayerStateSettingName(), retrievedState);
+    emit playerStateChanged(retrievedState);
+  }
+
 
   library_song_id_t retrievedCurrentId =
     newPlaylist["current_song"].toMap()["song"].toMap()["id"].value<library_song_id_t>();
-  if(retrievedCurrentId != currentSongId){
+  if(retrievedCurrentId != currentSongId && !clearingCurrentSong){
     QSqlQuery getSongQuery(
       "SELECT " + getLibFileColName() + ", " +
       getLibSongColName() + ", " +
-      getLibArtistColName() + " FROM " +
+      getLibArtistColName() + ", " + 
+      getLibDurationColName() + " FROM " +
       getActivePlaylistViewName() + " WHERE " + 
       getActivePlaylistLibIdColName() + " = " + QString::number(retrievedCurrentId) + ";", 
       database);
@@ -845,10 +940,12 @@ void DataStore::setActivePlaylist(const QVariantMap& newPlaylist){
       Logger::instance()->log("Got file, for manual song set");
       QString filePath = getSongQuery.value(0).toString();
       currentSongId = retrievedCurrentId;
+      QTime qtime(0, getSongQuery.value(3).toInt()/60, getSongQuery.value(3).toInt()%60);
       song_info_t toEmit = {
         Phonon::MediaSource(filePath),
         getSongQuery.value(1).toString(),
-        getSongQuery.value(2).toString()
+        getSongQuery.value(2).toString(),
+	qtime.toString("mm:ss")
       };
       emit manualSongChange(toEmit);
     }
@@ -903,12 +1000,15 @@ void DataStore::refreshActivePlaylist(){
   serverConnection->getActivePlaylist();
 }
 
+void DataStore::refreshParticipantList(){
+  serverConnection->getParticipantList();
+}
+
 
 void DataStore::onPlayerCreate(const player_id_t& issuedId){
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
   settings.setValue(getPlayerIdSettingName(), QVariant::fromValue(issuedId));
   serverConnection->setPlayerId(issuedId);
-  setPlayerState(getPlayingState());
   emit playerCreated();
 }
 
@@ -919,23 +1019,6 @@ void DataStore::onPlayerCreationFailed(const QString& errMessage, int /*errorCod
   emit playerCreationFailed(errMessage);
 }
 
-
-void DataStore::onPlayerStateChanged(const QString& newState){
-  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-  if(newState != settings.value(getPlayerStateSettingName())){
-    settings.setValue(getPlayerStateSettingName(), newState);
-    emit playerStateChanged(newState);
-  }
-
-  //If this player state change is the result of starting up the player for the first time
-  //we need to do a few things
-  if(!activePlaylistRefreshTimer->isActive()){
-    refreshActivePlaylist();
-    activePlaylistRefreshTimer->start();
-  }
-}
-
-
 void DataStore::onLibModError(
     const QString& errMessage, int errorCode, const QList<QNetworkReply::RawHeaderPair>& headers)
 {
@@ -972,6 +1055,13 @@ void DataStore::onSetVolumeFailed(
     reauthActions.insert(SET_CURRENT_VOLUME);
     initReauth();
   }
+  else{
+    emit setVolumeError(errMessage);
+  }
+}
+
+void DataStore::onNewParticipantList(const QVariantList& newParticipants){
+  emit newParticipantList(newParticipants);
 }
 
 
@@ -992,6 +1082,7 @@ void DataStore::onReauth(const QByteArray& ticketHash, const user_id_t& userId){
 }
 
 void DataStore::doReauthAction(const ReauthAction& action){
+  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
   switch(action){
     case SYNC_LIB:
       syncLibrary();
@@ -1008,16 +1099,38 @@ void DataStore::doReauthAction(const ReauthAction& action){
       serverConnection->modActivePlaylist(playlistIdsToAdd, playlistIdsToRemove);
       break;
     case SET_CURRENT_VOLUME:
-      QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
       serverConnection->setVolume((int)(getPlayerVolume() * 10));
       break;
+    case SET_PLAYER_STATE:
+      serverConnection->setPlayerState(getPlayerState());
+      break;
+    case SET_PLAYER_INACTIVE:
+      serverConnection->setPlayerState(getInactiveState());
+      break;
+    case SET_PLAYER_LOCATION:
+      serverConnection->setPlayerLocation(
+        settings.value(getAddressSettingName()).toString(),
+        settings.value(getCitySettingName()).toString(),
+        settings.value(getStateSettingName()).toString(),
+        settings.value(getZipCodeSettingName()).toString());
+      break;
+    case SET_PLAYER_PASSWORD:
+      serverConnection->setPlayerPassword(reauthPlayerPassword);
+      reauthPlayerPassword="";
+      break;
+    case REMOVE_PLAYER_PASSWORD:
+      serverConnection->removePlayerPassword();
+      break;
+    case CLEAR_CURRENT_SONG:
+      serverConnection->clearCurrentSong();
+      break;
   }
 }
 
 void DataStore::onAuthFail(const QString& /*errMessage*/){
-  isReauthing=false;
   Logger::instance()->log("BAD STUFF, BAD AUTH CREDS, BAD REAUTH");
-  //TODO need to do something here
+  setPasswordDirty();
+  emit hardAuthFailure();
 }
 
 void DataStore::initReauth(){
@@ -1052,42 +1165,51 @@ void DataStore::setDontShowPlaybackError(bool checked){
   settings.setValue(getDontShowPlaybackErrorSettingName(), checked);
 }
 
-
-void DataStore::saveCredentials(
-  const QString& username, const QString& password)
-{
+void DataStore::saveUsername(const QString& username){
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
   SimpleCrypt crypt = Utils::getCryptoObject();
   QString cryptUsername = crypt.encryptToString(username);
-  QString cryptPassword = crypt.encryptToString(password);
-  settings.setValue(getHasValidCredsSettingName(), true);
   settings.setValue(getUsernameSettingName(), cryptUsername);
-  settings.setValue(getPasswordSettingName(), cryptPassword);
 }
 
-void DataStore::setCredentialsDirty(){
-  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-  settings.setValue(getHasValidCredsSettingName(), false);
-}
-
-bool DataStore::hasValidSavedCredentials(){
-  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-  return settings.value(getHasValidCredsSettingName()).toBool();
-}
-
-void DataStore::getSavedCredentials(QString* username, QString* password){
+QString DataStore::getSavedUsername(){
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
   SimpleCrypt crypt = Utils::getCryptoObject();
   QString encryptedUsername = settings.value(getUsernameSettingName()).toString();
-  QString encryptedPassword = settings.value(getPasswordSettingName()).toString();
-  *username = crypt.decryptToString(encryptedUsername);
-  *password = crypt.decryptToString(encryptedPassword);
+  return encryptedUsername == "" ? "" : crypt.decryptToString(encryptedUsername);
 }
 
-void DataStore::clearSavedCredentials(){
+
+
+void DataStore::savePassword(const QString& password)
+{
   QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-  settings.setValue(getHasValidCredsSettingName(), false);
-  settings.setValue(getUsernameSettingName(), "");
+  SimpleCrypt crypt = Utils::getCryptoObject();
+  QString cryptPassword = crypt.encryptToString(password);
+  settings.setValue(getHasValidSavedPasswordSettingName(), true);
+  settings.setValue(getPasswordSettingName(), cryptPassword);
+}
+
+void DataStore::setPasswordDirty(){
+  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
+  settings.setValue(getHasValidSavedPasswordSettingName(), false);
+}
+
+bool DataStore::hasValidSavedPassword(){
+  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
+  return settings.value(getHasValidSavedPasswordSettingName()).toBool();
+}
+
+QString DataStore::getSavedPassword(){
+  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
+  SimpleCrypt crypt = Utils::getCryptoObject();
+  QString encryptedPassword = settings.value(getPasswordSettingName()).toString();
+  return crypt.decryptToString(encryptedPassword);
+}
+
+void DataStore::clearSavedPassword(){
+  QSettings settings(QSettings::UserScope, getSettingsOrg(), getSettingsApp());
+  settings.setValue(getHasValidSavedPasswordSettingName(), false);
   settings.setValue(getPasswordSettingName(), "");
 }
 
