@@ -51,7 +51,13 @@ public:
     GET_ACTIVE_PLAYLIST,
     SET_CURRENT_SONG,
     MOD_PLAYLIST,
-    SET_CURRENT_VOLUME
+    SET_CURRENT_VOLUME,
+    SET_PLAYER_STATE,
+    SET_PLAYER_INACTIVE,
+    SET_PLAYER_LOCATION,
+    SET_PLAYER_PASSWORD,
+    REMOVE_PLAYER_PASSWORD,
+    CLEAR_CURRENT_SONG
   };
 
   /**
@@ -61,6 +67,7 @@ public:
     Phonon::MediaSource source;
     QString title;
     QString artist;
+    QString duration;
   } song_info_t;
 
   //@}
@@ -86,6 +93,26 @@ public:
 
   //@}
 
+  /** @name Accessors */
+  //@{
+
+  /**
+   * \brief Checks to see if a particular song is already in 
+   * the library and not deleted.
+   *
+   * @param fileName The file to be checked.
+   * @return True if the file is already in the library, false 
+   * otherwise.
+   */
+  bool alreadyHaveSongInLibrary(const QString& fileName) const;
+
+  inline library_song_id_t getCurrentSongId() const{
+    return currentSongId;
+  }
+
+  //@}
+
+
   /** @name Modifiers */
   //@{
 
@@ -100,6 +127,11 @@ public:
     const QList<Phonon::MediaSource>& songs, 
     QProgressDialog* progress=0);
 
+  /**
+   * \brief Clears the current song that is playing.
+   */
+  void clearCurrentSong();
+
   /**
    * \brief Removes the password on the player.
    */
@@ -124,22 +156,13 @@ public:
     const QString& streetAddress,
     const QString& city,
     const QString& state,
-    int zipcode
+    const QString& zipcode
   );
 
   /**
-   * \brief Set player name.
-   *
-   * \param The new name the player should have.
+   * \brief Sets the player as inactive.
    */
-  void setPlayerName(const QString& newName);
-
-  /**
-   * \brief Set player state.
-   *
-   * \param State to which the player should be set.
-   */
-  void setPlayerState(const QString& newState);
+  void setPlayerInactive();
 
   /**
    * \brief Removes the given songs from the music library. 
@@ -175,7 +198,7 @@ public:
    *
    * @return The id of the player.
    */
-  inline const player_id_t getPlayerId() const{
+  inline player_id_t getPlayerId() const{
     QSettings settings(
       QSettings::UserScope, getSettingsOrg(), getSettingsApp());
     return settings.value(getPlayerIdSettingName()).value<player_id_t>();
@@ -230,18 +253,7 @@ public:
   inline bool hasPlayerPassword() const{
     QSettings settings(
       QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-    return settings.contains(getPlayerPasswordSettingName());
-  }
-
-  /**
-   * \brief Retreives the password for the player.
-   *
-   * @return The password for the player, if none is set a blank string is returned.
-   */
-  inline QString getPlayerPassword() const{
-    QSettings settings(
-      QSettings::UserScope, getSettingsOrg(), getSettingsApp());
-    return settings.value(getPlayerPasswordSettingName()).toString();
+    return settings.value(getHasPlayerPasswordSettingName(), false).toBool();
   }
 
   /**
@@ -339,44 +351,56 @@ public:
    *
    * \return True if the player has an id, false otherwise.
    */
-  bool hasPlayerId() const{
+  static bool hasPlayerId(){
     QSettings settings(
       QSettings::UserScope, getSettingsOrg(), getSettingsApp());
     return -1 != settings.value(getPlayerIdSettingName(), -1);
   }
 
   /**
-   * \brief Saves the given credentials to persistent storage in a secure manner.
+   * \brief Saves the given password to persistent storage in a "secure" manner.
    *
-   * @param username The username to save.
    * @param password The password to save.
    */
-  static void saveCredentials(const QString& username, const QString& password);
+  static void savePassword(const QString& password);
 
   /**
    * \brief Marks the current saved credentials as invalid.
    */
-  static void setCredentialsDirty();
+  static void setPasswordDirty();
 
   /**
-   * \brief Determines whether or not the currently saved credentials are valid.
+   * \brief Determines whether or not the currently saved password is valid.
    *
-   * @return True if the currently saved credentials are valide, false otherwise.
+   * @return True if the currently saved password is valid, false otherwise.
    */
-  static bool hasValidSavedCredentials();
+  static bool hasValidSavedPassword();
 
   /**
-   * \brief Retrieves the currently saved credentials.
+   * \brief Retrieves the currently saved password.
    *
-   * @param username Pointer to the QString where the retreived username should be put.
-   * @param password Pointer to the QString where the retreived password should be put.
+   * @return The saved password.
    */
-  static void getSavedCredentials(QString* username, QString* password);
+  static QString getSavedPassword();
 
   /**
    * \brief Deletes all the saved credentials.
    */
-  static void clearSavedCredentials();
+  static void clearSavedPassword();
+
+  /**
+   * \brief Saves the given username to persistent storage
+   *
+   * \param username Username to save to presistent storage.
+   */
+  static void saveUsername(const QString& username);
+
+  /**
+   * \brief Retrieves the last username that was saved to presistent storage.
+   *
+   * \return The last username that was saved to persistent storage.
+   */
+  static QString getSavedUsername();
 
   static bool getDontShowPlaybackErrorSetting();
 
@@ -765,13 +789,13 @@ public:
   }
 
   /**
-   * \brief Name of the setting used to store the password being used for the player.
+   * \brief Name of the setting used to store whether or not the player has a password.
    *
-   * @return Name of the setting used to store the password being used for the player.
+   * @return Name of the setting used to store whether or not the player has a password.
    */
-  static const QString& getPlayerPasswordSettingName(){
-    static const QString playerPasswordSettingName = "playerPassword";
-    return playerPasswordSettingName;
+  static const QString& getHasPlayerPasswordSettingName(){
+    static const QString hasPlayerPasswordSettingName = "hasPlayerPassword";
+    return hasPlayerPasswordSettingName;
   }
 
   /**
@@ -875,6 +899,15 @@ public:
 //@{
 public slots:
 
+  /**
+   * \brief Starts the datastore automatically refreshing the playlist.
+   */
+  void startPlaylistAutoRefresh();
+
+  /**
+   * \brief Starts the datastore automatically refreshing the list of participants.
+   */
+  void startParticipantsAutoRefresh();
 
   /**
    * \brief Syncs the current state of the library with the server.
@@ -896,6 +929,9 @@ public slots:
    */
   void refreshActivePlaylist();
 
+  /** \brief Refresh the participants list. */
+  void refreshParticipantList();
+
   /**
    * \brief Adds the given song to the active playlist.
    *
@@ -943,7 +979,7 @@ public slots:
     const QString& streetAddress,
     const QString& city,
     const QString& state,
-    const int& zipcode);
+    const QString& zipcode);
 
   /** 
    * \brief Sets the current song to the speicified song.
@@ -953,11 +989,11 @@ public slots:
   void setCurrentSong(const library_song_id_t& songToPlay);
 
   /** 
-   * \brief Changes the volume without emtting a signal noting that the volume has changed.
+   * \brief Sets the volume for the player.
    *
    * @param newVolume The new player volume.
    */
-  void changeVolumeSilently(qreal newVolume);
+  void setVolume(qreal newVolume);
 
   /** \brief Determines whether or not the library has unsynced songs.*/
   bool hasUnsyncedSongs() const;
@@ -973,7 +1009,38 @@ signals:
 //@{
 
   /**
-   * \brief Emitted when the player's password has been removed.
+   * \brief Emitted when the given user credentials do not work with the server.
+   */
+  void hardAuthFailure();
+
+  /**
+   * \brief Emitted when the player state was succesfully changed to inactive.
+   */
+  void playerSuccessfullySetInactive();
+
+  /**
+   * \brief Emitted when there was an error setting the player inactive.
+   *
+   * @param errMessage A message descibing the error.
+   */
+  void playerSetInactiveError(const QString& errMessage);
+
+  /**
+   * \brief Emitted when there was an error setting the player to "playing".
+   *
+   * @param errMessage A message descibing the error.
+   */
+  void playPlayerError(const QString& errMessage);
+
+  /**
+   * \brief Emitted when there was an error setting the player to "paused".
+   *
+   * @param errMessage A message descibing the error.
+   */
+  void pausePlayerError(const QString& errMessage);
+
+  /**
+   * \brief Emitted when the player's password has been removed by a component of this client.
    */
   void playerPasswordRemoved();
 
@@ -985,7 +1052,7 @@ signals:
   void playerPasswordRemoveError(const QString& errMessage);
 
   /**
-   * \brief Emitted when the player's password is set.
+   * \brief Emitted when the player's password is set by a compenent of this client.
    */
   void playerPasswordSet();
 
@@ -997,7 +1064,7 @@ signals:
   void playerPasswordSetError(const QString& errMessage);
 
   /**
-   * \brief Emitted when the players location has been set.
+   * \brief Emitted when the player's location is set by a component of this client.
    */
   void playerLocationSet();
 
@@ -1008,20 +1075,6 @@ signals:
    */
   void playerLocationSetError(const QString& errMessage);
 
-  /**
-   * \brief Emitted when the players name is succesfully changed.
-   *
-   * \param newName The new name the player was set to.
-   */
-  void playerNameChanged(const QString& newName);
-
-  /**
-   * \brief Emitted when changing the players name failes.
-   *
-   * \param errMessage An error message describing the failure.
-   */
-  void playerNameChangeError(const QString& errMessage);
-
   /**
    * \brief Emitted when the library table is modified.
    */
@@ -1052,10 +1105,15 @@ signals:
   void playerCreationFailed(const QString errMessage);
 
   /**
-   * \brief Emitted when the active playlist is modified.
+   * \brief Emitted when the active playlist is modified by the server.
    */
   void activePlaylistModified();
 
+  /**
+   * \brief Emitted when the participant list retrieved from server.
+   */
+  void newParticipantList(const QVariantList& newParticipants);
+
   /**
    * \brief Emitted when the current song is manually changed.
    *
@@ -1064,19 +1122,33 @@ signals:
   void manualSongChange(DataStore::song_info_t newSong);
 
   /**
-   * \brief Emitted when the players state is changed.
+   * \brief Emitted when the players state is changed by the server.
    *
    * \param newState The new state of the player.
    */
   void playerStateChanged(const QString& newState);
 
   /**
-   * \brief Emitted when the volume of the player is changed.
+   * \brief Emitted when the volume of the player is changed by the server.
    *
    * @param newVolume The new volume of the player.
    */
   void volumeChanged(qreal newVolume);
 
+  /**
+   * \brief Emitted when changing the volume on the server failed.
+   *
+   * @param errMessage A message describing the error.
+   */
+  void setVolumeError(const QString& errMessage);
+
+  /**
+   * \brief Emitted when clearing the current song on ther server failed.
+   *
+   * @param errMessage A message describing the error.
+   */
+  void clearCurrentSongError(const QString& errMessage);
+
 //@}
 
 private:
@@ -1093,6 +1165,9 @@ private:
   /** \brief Timer used to refresh the active playlist. */
   QTimer *activePlaylistRefreshTimer;
 
+  /** \brief Timer used to refresh the list of participants. */
+  QTimer *participantRefreshTimer;
+
   /** \brief Current username being used by the client */
   QString username;
 
@@ -1102,9 +1177,31 @@ private:
   /** \brief A set of actions to be performed once the client has succesfully reauthenticated. */
   QSet<ReauthAction> reauthActions;
 
+  /** 
+   * \brief If there is an auth error during the setting of a player password, this is the password that
+   * should be set on the player after the reauth is complete.
+   */
+  QString reauthPlayerPassword;
+
   /** \brief Whether or not the client is currently reauthenticating. */
   bool isReauthing;
 
+  /**
+   * \brief Whether or not the client is currently setting the player's playback state.
+   *
+   * This is used to help us ignore playback states that are different from our own while
+   * we're in the process of changing the playback state.
+   */
+  bool changingPlayerState;
+
+  /**
+   * \brief Whether or not the client is currently setting the current song as cleared.
+   *
+   * This is used to help us ignore current song changes while we're clearing the current
+   * song.
+   */
+  bool clearingCurrentSong;
+
   /** \brief The current song being played. */
   library_song_id_t currentSongId;
 
@@ -1122,6 +1219,13 @@ private:
   /** \brief Does initial database setup */
   void setupDB();
 
+  /**
+   * \brief Set player state.
+   *
+   * \param State to which the player should be set.
+   */
+  void setPlayerState(const QString& newState);
+
   /**
    * \brief Deletes a single song from the active playlist.
    *
@@ -1154,6 +1258,7 @@ private:
    */
   void addSongToLibrary(const Phonon::MediaSource& song, QSqlQuery& addQuery);
 
+  
   /**
    * \brief Gets the value of a header.
    *
@@ -1330,10 +1435,10 @@ private:
    *
    * @return Name of the setting used to store whether or not the current credentials are valid.
    */
-  static const QString& getHasValidCredsSettingName(){
-    static const QString hasValidSavedCredentialsSettingName = 
-      "has_valid_creds";
-    return hasValidSavedCredentialsSettingName;
+  static const QString& getHasValidSavedPasswordSettingName(){
+    static const QString hasValidSavedPasswordSettingName = 
+      "has_valid_password";
+    return hasValidSavedPasswordSettingName;
   }
 
  //@}
@@ -1343,9 +1448,44 @@ private:
 private slots:
 
   /**
-   * \brief Preforms appropriate tasks when a player's password has been removed.
+   * \brief Performs appropriate tasks when the player's state has been succesfully changed on the
+   * server.
+   *
+   * \param state The state the player was changed to on the server.
    */
-  void onPlayerPasswordRemoved();
+  void onPlayerStateSet(const QString& state);
+
+  /**
+   * \brief Performs appropriate tasks when the player's state has been succesfully changed on the
+   * server.
+   *
+   * \param state The state which was attempted to be set on the player.
+   * \param errMessage A message describing the error.
+   * \param errorCode HTTP error code describing error.
+   * \param headers HTTP headers accompianing in the error response.
+   */
+  void onPlayerStateSetError(
+    const QString& state,
+    const QString& errMessage,
+    int errorCode,
+    const QList<QNetworkReply::RawHeaderPair>& headers);
+
+  /**
+   * \brief Performs appropriate tasks when the current song has been succesfully cleared.
+   */
+  void onCurrentSongCleared();
+
+  /**
+   * \brief Preforms appropriate tasks when there was an error clearing the current song.
+   *
+   * \param errMessage A message describing the error.
+   * \param errorCode HTTP error code describing error.
+   * \param headers HTTP headers accompianing in the error response.
+   */
+  void onCurrentSongClearError(
+    const QString& errMessage,
+    int errorCode,
+    const QList<QNetworkReply::RawHeaderPair>& headers);
 
   /**
    * \brief Preforms appropriate tasks when there was an error removing the player's password.
@@ -1359,41 +1499,20 @@ private slots:
     int errorCode,
     const QList<QNetworkReply::RawHeaderPair>& headers);
 
-  /**
-   * \brief Preforms appropriate tasks when a player's password is set.
-   *
-   * \brief password The password that has been set on the server.
-   */
-  void onPlayerPasswordSet(const QString& password);
-
   /**
    * \brief Preforms appropriate tasks when there was an error setting the player's location.
    *
+   * \param attemptedPassword The password that was attempted to be set on the server.
    * \param errMessage A message describing the error.
    * \param errorCode HTTP error code describing error.
    * \param headers HTTP headers accompianing in the error response.
    */
   void onPlayerPasswordSetError(
+    const QString& attemptedPassword,
     const QString& errMessage,
     int errorCode,
     const QList<QNetworkReply::RawHeaderPair>& headers);
 
-
-  /**
-   * \brief Preforms appropriate tasks when a players location was succesfully set.
-   *
-   * \brief streetAddress The street address of location that was set for the player.
-   * \brief city The city of location that was set for the player.
-   * \brief state The state of location that was set for the player.
-   * \brief zipcode The zipcode of location that was set for the player.
-   */
-  void onPlayerLocationSet(
-    const QString& streetAddress,
-    const QString& city,
-    const QString& state,
-    int zipcode
-  );
-
   /**
    * \brief Preforms appropriate tasks when there was an error setting the player's location.
    *
@@ -1406,26 +1525,6 @@ private slots:
     int errorCode,
     const QList<QNetworkReply::RawHeaderPair>& headers);
 
-  /**
-   * \brief Preforms appropriate tasks when a players name was succesfully changed on the
-   * server.
-   *
-   * \param newName The new name of the player.
-   */
-  void onPlayerNameChanged(const QString& newName);
-
-  /**
-   * \brief Preforms appropriate tasks when there was an error changing the player name.
-   *
-   * \param errMessage A message describing the error.
-   * \param errorCode HTTP error code describing error.
-   * \param headers HTTP headers accompianing in the error response.
-   */
-  void onPlayerNameChangeError(
-    const QString& errMessage,
-    int errorCode,
-    const QList<QNetworkReply::RawHeaderPair>& headers);
-
   /**
    * \brief Sets the sync status of a library song to synced.
    *
@@ -1499,12 +1598,6 @@ private slots:
   void onPlayerCreationFailed(const QString& errMessage, int errorCode,
           const QList<QNetworkReply::RawHeaderPair>& headers);
 
-  /**
-   * \brief Takes appropriate action when the player state is changed.
-   */
-  void onPlayerStateChanged(const QString& newState);
-
-
   /**
    * \brief Takes appropriate action when modifiying the library on the server fails.
    *
@@ -1578,6 +1671,14 @@ private slots:
     int errorCode,
     const QList<QNetworkReply::RawHeaderPair>& headers);
 
+  /**
+   * \brief Takes appropriate action when the particpants list is retrieved from the server.
+   *
+   * \param newParticipants The new list of participants.
+   */
+  void onNewParticipantList(const QVariantList& newParticipants);
+
+
   //@}
 
 
