@@ -79,8 +79,10 @@ void UDJServerConnection::modLibContents(const QVariantList& songsToAdd,
     + "&to_delete=" + deleteJSON;
   Logger::instance()->log("Lib mod payload: " + QString::fromUtf8(payload));
   QNetworkReply *reply = netAccessManager->post(modRequest, payload);
+  Logger::instance()->log("Issued request" + QString::fromUtf8(payload));
   reply->setProperty(getSongsAddedPropertyName(), addJSON);
   reply->setProperty(getSongsDeletedPropertyName(), deleteJSON);
+  Logger::instance()->log("Set reply properties" + QString::fromUtf8(payload));
 }
 
 void UDJServerConnection::createPlayer(
@@ -97,7 +99,7 @@ void UDJServerConnection::createPlayer(
   const QString& streetAddress,
   const QString& city,
   const QString& state,
-  const int& zipcode)
+  const QString& zipcode)
 {
   const QByteArray playerJSON = JSONHelper::getCreatePlayerJSON(playerName, password,
       streetAddress, city, state, zipcode);
@@ -131,16 +133,17 @@ void UDJServerConnection::setPlayerLocation(
   const QString& streetAddress,
   const QString& city,
   const QString& state,
-  int zipcode
+  const QString& zipcode
 )
 {
   QNetworkRequest setLocationRequest(getPlayerLocationUrl());
   setLocationRequest.setRawHeader(getTicketHeaderName(), ticket_hash);
   QUrl params;
   params.addQueryItem("address", streetAddress);
-  params.addQueryItem("city", city);
-  params.addQueryItem("state", state);
-  params.addQueryItem("zipcode", QString::number(zipcode));
+  params.addQueryItem("locality", city);
+  params.addQueryItem("region", state);
+  params.addQueryItem("postal_code", zipcode);
+  params.addQueryItem("country", "United States");
   QByteArray payload = params.encodedQuery();
   QNetworkReply *reply = netAccessManager->post(setLocationRequest, payload);
   reply->setProperty(getLocationAddressPropertyName(), streetAddress);
@@ -149,16 +152,6 @@ void UDJServerConnection::setPlayerLocation(
   reply->setProperty(getLocationZipcodePropertyName(), zipcode);
 }
 
-void UDJServerConnection::setPlayerName(const QString& newName){
-  QNetworkRequest setNameRequest(getPlayerNameUrl());
-  setNameRequest.setRawHeader(getTicketHeaderName(), ticket_hash);
-  QUrl params;
-  params.addQueryItem("name", newName);
-  QByteArray payload = params.encodedQuery();
-  QNetworkReply *reply = netAccessManager->post(setNameRequest, payload);
-  reply->setProperty(getPlayerNamePropertyName(), newName);
-}
-
 void UDJServerConnection::getActivePlaylist(){
   QNetworkRequest getActivePlaylistRequest(getActivePlaylistUrl());
   getActivePlaylistRequest.setRawHeader(getTicketHeaderName(), ticket_hash);
@@ -212,6 +205,20 @@ void UDJServerConnection::setPlayerState(const QString& newState){
   reply->setProperty(getStatePropertyName(), newState);
 }
 
+void UDJServerConnection::clearCurrentSong(){
+  Logger::instance()->log("Clearing current song");
+  QNetworkRequest clearCurrentSongRequest(getCurrentSongUrl());
+  clearCurrentSongRequest.setRawHeader(getTicketHeaderName(), ticket_hash);
+  /*QNetworkReply *reply =*/ netAccessManager->deleteResource(clearCurrentSongRequest);
+}
+
+
+void UDJServerConnection::getParticipantList(){
+  QNetworkRequest getParticipantListRequest(getParticipantsUrl());
+  getParticipantListRequest.setRawHeader(getTicketHeaderName(), ticket_hash);
+  /*QNetworkReply *reply =*/ netAccessManager->get(getParticipantListRequest);
+}
+
 void UDJServerConnection::recievedReply(QNetworkReply *reply){
   if(reply->request().url().path() == getAuthUrl().path()){
     handleAuthReply(reply);
@@ -219,15 +226,22 @@ void UDJServerConnection::recievedReply(QNetworkReply *reply){
   else if(reply->request().url().path() == getPlayerStateUrl().path()){
     handleSetStateReply(reply);
   }
-  else if(isPlayerCreateUrl(reply->request().url().path())){
+  else if(reply->request().url().path() == getCreatePlayerUrl().path()){
     handleCreatePlayerReply(reply);
   }
   else if(isGetActivePlaylistReply(reply)){
     handleReceivedActivePlaylist(reply);
   }
-  else if(reply->request().url().path() == getCurrentSongUrl().path()){
+  else if(reply->request().url().path() == getCurrentSongUrl().path() && 
+      reply->operation() == QNetworkAccessManager::PostOperation)
+  {
     handleReceivedCurrentSongSet(reply);
   }
+  else if(reply->request().url().path() == getCurrentSongUrl().path() && 
+      reply->operation() == QNetworkAccessManager::DeleteOperation)
+  {
+    handleRecievedClearCurrentSong(reply);
+  }
   else if(reply->request().url().path() == getLibModUrl().path()){
     handleReceivedLibMod(reply);
   }
@@ -237,9 +251,6 @@ void UDJServerConnection::recievedReply(QNetworkReply *reply){
   else if(isModActivePlaylistReply(reply)){
     handleReceivedPlaylistMod(reply);
   }
-  else if(reply->request().url().path() == getPlayerNameUrl().path()){
-    handleNameSetReply(reply);
-  }
   else if(reply->request().url().path() == getPlayerLocationUrl().path()){
     handleLocationSetReply(reply);
   }
@@ -249,6 +260,9 @@ void UDJServerConnection::recievedReply(QNetworkReply *reply){
   else if(isPasswordRemoveReply(reply)){
     handlePlayerPasswordRemoveReply(reply);
   }
+  else if(reply->request().url().path() == getParticipantsUrl().path()){
+    handleParticipantsResponse(reply);
+  }
   else{
     Logger::instance()->log("Received unknown response");
     Logger::instance()->log(reply->request().url().path());
@@ -266,6 +280,9 @@ void UDJServerConnection::handleAuthReply(QNetworkReply* reply){
   else if(reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) == 401){
     emit authFailed(tr("Incorrect Username and password"));
   }
+  else if(reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) == 501){
+    emit authFailed(tr("Your version of the UDJ player is out of date. Please check www.udjplayer.com for an update."));
+  }
   else{
     QByteArray responseData = reply->readAll();
     QString responseString = QString::fromUtf8(responseData);
@@ -276,12 +293,48 @@ void UDJServerConnection::handleAuthReply(QNetworkReply* reply){
   }
 }
 
+void UDJServerConnection::handleRecievedClearCurrentSong(QNetworkReply *reply){
+  if(isResponseType(reply, 200)){
+    emit currentSongCleared();
+  }
+  else{
+    QString responseData = QString(reply->readAll());
+    Logger::instance()->log("Clear current song failed " + responseData);
+    emit currentSongClearError(
+      responseData,
+      reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt(),
+      reply->rawHeaderPairs());
+  }
+
+}
+
+void UDJServerConnection::handleParticipantsResponse(QNetworkReply *reply){
+  if(isResponseType(reply, 200)){
+    emit newParticipantList(JSONHelper::getParticipantListFromJSON(reply));
+  }
+  else{
+    QString responseData = QString(reply->readAll());
+    Logger::instance()->log("Participan get error " + responseData);
+    emit getParticipantsError(
+      responseData,
+      reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt(),
+      reply->rawHeaderPairs());
+  }
+
+}
+
 void UDJServerConnection::handleSetStateReply(QNetworkReply *reply){
   if(isResponseType(reply, 200)){
     emit playerStateSet(reply->property(getStatePropertyName()).toString());
   }
   else{
-    //TODO handle error
+    QString responseData = QString(reply->readAll());
+    Logger::instance()->log("Player state set error " + responseData);
+    emit playerStateSetError(
+      reply->property(getStatePropertyName()).toString(),
+      responseData,
+      reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt(),
+      reply->rawHeaderPairs());
   }
 }
 
@@ -308,24 +361,7 @@ void UDJServerConnection::handlePlayerPasswordSetReply(QNetworkReply *reply){
     QString responseData = QString(reply->readAll());
     Logger::instance()->log("Set player password error " + responseData);
     emit playerPasswordSetError(
-      responseData,
-      reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt(),
-      reply->rawHeaderPairs());
-  }
-}
-
-void UDJServerConnection::handleNameSetReply(QNetworkReply *reply){
-  if(isResponseType(reply, 200)){
-    emit playerNameChanged(reply->property(getPlayerNamePropertyName()).toString());
-  }
-  else if(isResponseType(reply, 409)){
-    Logger::instance()->log("Change player name got 409");
-    emit playerNameChangeError("Name already in use.", 409, reply->rawHeaderPairs());
-  }
-  else{
-    QString responseData = QString(reply->readAll());
-    Logger::instance()->log("Change player name got error " + responseData);
-    emit playerNameChangeError(
+      reply->property(getPlayerPasswordPropertyName()).toString(),
       responseData,
       reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt(),
       reply->rawHeaderPairs());
@@ -338,7 +374,7 @@ void UDJServerConnection::handleLocationSetReply(QNetworkReply *reply){
       reply->property(getLocationAddressPropertyName()).toString(),
       reply->property(getLocationCityPropertyName()).toString(),
       reply->property(getLocationStatePropertyName()).toString(),
-      reply->property(getLocationZipcodePropertyName()).toInt());
+      reply->property(getLocationZipcodePropertyName()).toString());
   }
   else{
     QString responseData = QString(reply->readAll());
@@ -352,6 +388,7 @@ void UDJServerConnection::handleLocationSetReply(QNetworkReply *reply){
 
 void UDJServerConnection::handleReceivedLibMod(QNetworkReply *reply){
   if(isResponseType(reply, 200)){
+    Logger::instance()->log("got good lib mod reply");
     QVariant songsAdded = reply->property(getSongsAddedPropertyName());
     QVariant songsDeleted = reply->property(getSongsDeletedPropertyName());
     QSet<library_song_id_t> addedIds = JSONHelper::getLibIds(songsAdded.toByteArray());
@@ -360,10 +397,6 @@ void UDJServerConnection::handleReceivedLibMod(QNetworkReply *reply){
     QSet<library_song_id_t> allSynced = addedIds.unite(deletedIds);
     emit libSongsSyncedToServer(allSynced);
   }
-  else if(isResponseType(reply, 409)){
-    QSet<library_song_id_t> alreadyExistingIds = JSONHelper::convertLibIdArray(reply->readAll());
-    emit libSongsSyncedToServer(alreadyExistingIds);
-  }
   else{
     Logger::instance()->log("Got bad lib mod");
     QByteArray response = reply->readAll();
@@ -463,45 +496,40 @@ QUrl UDJServerConnection::getCurrentSongUrl() const{
 }
 
 QUrl UDJServerConnection::getCreatePlayerUrl() const{
-  return QUrl(getServerUrlPath()+ "users/" + QString::number(user_id) + "/players/player");
+  return QUrl(getServerUrlPath()+ "players/player");
 }
 
 QUrl UDJServerConnection::getPlayerStateUrl() const{
-  return QUrl(getServerUrlPath()+ "users/" + QString::number(user_id) + "/players/" + 
+  return QUrl(getServerUrlPath()+ "players/" + 
       QString::number(playerId) + "/state");
 }
 
 QUrl UDJServerConnection::getLibModUrl() const{
-  return QUrl(getServerUrlPath()+ "users/" + QString::number(user_id) + "/players/" + 
+  return QUrl(getServerUrlPath()+ "/players/" + 
       QString::number(playerId) + "/library");
 }
 
 QUrl UDJServerConnection::getVolumeUrl() const{
-  return QUrl(getServerUrlPath()+ "users/" + QString::number(user_id) + "/players/" + 
+  return QUrl(getServerUrlPath()+ "players/" + 
       QString::number(playerId) + "/volume");
 }
 
-QUrl UDJServerConnection::getPlayerNameUrl() const{
-  return QUrl(getServerUrlPath()+ "users/" + QString::number(user_id) + "/players/" + 
-      QString::number(playerId) + "/name");
-}
-
 QUrl UDJServerConnection::getPlayerLocationUrl() const{
-  return QUrl(getServerUrlPath()+ "users/" + QString::number(user_id) + "/players/" + 
+  return QUrl(getServerUrlPath()+ "players/" + 
       QString::number(playerId) + "/location");
 }
 
 QUrl UDJServerConnection::getPlayerPasswordUrl() const{
-  return QUrl(getServerUrlPath()+ "users/" + QString::number(user_id) + "/players/" + 
+  return QUrl(getServerUrlPath()+ "players/" + 
       QString::number(playerId) + "/password");
 }
 
-
-
-bool UDJServerConnection::isPlayerCreateUrl(const QString& path) const{
-  return (path == "/udj/users/" + QString::number(user_id) + "/players/player");
+QUrl UDJServerConnection::getParticipantsUrl() const{
+  return QUrl(getServerUrlPath()+ "players/"+QString::number(playerId)+"/users");
 }
 
+
+
 bool UDJServerConnection::isResponseType(QNetworkReply *reply, int code){
   return reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) == code;
 }
@@ -531,4 +559,6 @@ bool UDJServerConnection::isPasswordRemoveReply(const QNetworkReply *reply) cons
 }
 
 
+
+
 }//end namespace
