@@ -111,7 +111,7 @@ public slots:
     const QString& streetAddress,
     const QString& city,
     const QString& state,
-    int zipcode
+    const QString& zipcode
   );
 
   /**
@@ -126,13 +126,6 @@ public slots:
    */
   void setPlayerPassword(const QString& newPassword);
 
-  /**
-   * \brief Tells the server to change the players name to the new name.
-   *
-   * \param newName New name for the player.
-   */
-  void setPlayerName(const QString& newName);
-
   /**
    * \brief Sets the player state.
    *
@@ -170,7 +163,7 @@ public slots:
     const QString& streetAddress,
     const QString& city,
     const QString& state,
-    const int& zipcode);
+    const QString& zipcode);
 
   /**
    * \brief Creates a player on the server using the given JSON payload.
@@ -208,6 +201,16 @@ public slots:
    */
   void setVolume(int newVolume);
 
+  /**
+   * \brief Tells the server that current song has finished playing.
+   */
+  void clearCurrentSong();
+
+  /**
+   * \brief Retrieves a list of active participants from the server.
+   */
+  void getParticipantList();
+
   //@}
 
 signals:
@@ -259,11 +262,13 @@ signals:
   /**
    * \brief Emitted when there was an error setting the player's password.
    *
+   * @param attemptedPassword The password that was attempted to be set on the player.
    * @param errMessage A message describing the error.
    * @param errorCode The http status code that describes the error.
    * @param headers The headers from the http response that indicated a failure.
    */
   void playerPasswordSetError(
+    const QString& attemptedPassword,
     const QString& errMessage,
     int errorCode,
     const QList<QNetworkReply::RawHeaderPair>& headers);
@@ -280,7 +285,7 @@ signals:
     const QString& streetAddress,
     const QString& city,
     const QString& state,
-    int zipcode);
+    const QString&  zipcode);
 
   /**
    * \brief Emitted when there was an error setting the player's location.
@@ -313,6 +318,20 @@ signals:
     int errorCode,
     const QList<QNetworkReply::RawHeaderPair>& headers);
 
+  /**
+   * \brief Emitted when the current song was succesfully cleared on the server.
+   */
+  void currentSongCleared();
+
+  /**
+   * \brief Emitted when there is an error clearing the current song on the server.
+   */
+  void currentSongClearError(
+    const QString& errMessage,
+    int errorCode,
+    const QList<QNetworkReply::RawHeaderPair>& headers);
+
+
 
   /**
    * \brief Emitted when the state on the player was changed.
@@ -321,6 +340,21 @@ signals:
    */
   void playerStateSet(const QString& newState);
 
+  /**
+   * \brief Emitted when there was an error changing the players state.
+   *
+   * @param desiredState The state which failed to be set on the player.
+   * @param errMessage A message describing the error.
+   * @param errorCode The http status code that describes the error.
+   * @param headers The headers from the http response that indicated a failure.
+   */
+  void playerStateSetError(
+    const QString& desiredState,
+    const QString& errMessage,
+    int errorCode,
+    const QList<QNetworkReply::RawHeaderPair>& headers);
+
+
   /**
    * \brief Emitted when a set of songs was succesfully synced on the server.
    *
@@ -360,6 +394,8 @@ signals:
   /**
    * \brief Emitted when a new version of the active playlist is retrieved from
    * the server.
+   *
+   * @param newPlaylist The new playlist that was retreived from the server.
    */
   void newActivePlaylist(const QVariantMap& newPlaylist);
 
@@ -375,6 +411,25 @@ signals:
     int errorCode,
     const QList<QNetworkReply::RawHeaderPair>& headers);
 
+  /**
+   * \brief Emitted when a new version of the participants list is retrieved from the server.
+   *
+   * @param newParticipants The new list of participants that was retrieved from the server.
+   */
+  void newParticipantList(const QVariantList& newParticipants);
+
+  /**
+   * \brief Emitted when there was an error getting the list of participants from the server.
+   *
+   * @param errMessage A message describing the error.
+   * @param errorCode The http status code that describes the error.
+   * @param headers The headers from the http response that indicated a failure.
+   */
+  void getParticipantsError(
+    const QString& errMessage,
+    int errorCode,
+    const QList<QNetworkReply::RawHeaderPair>& headers);
+
   /**
    * \brief Emitted when the current song that the player is playing is
    * succesfully set on the server.
@@ -559,16 +614,23 @@ private:
   /**
    * \brief Handle a response from the server regarding the setting of the player location.
    *
-   * @param reply Response from the server.
+   * @param reply response from the server.
    */
   void handleLocationSetReply(QNetworkReply *reply);
 
   /**
-   * \brief Handle a response from the server regarding the setting of the player name.
+   * \brief Handles a response from the server regarding the clearing of the current song.
    *
-   * @param reply Response from the server.
+   * @param reply The response from the server.
    */
-  void handleNameSetReply(QNetworkReply *reply);
+  void handleRecievedClearCurrentSong(QNetworkReply *reply);
+
+  /**
+   * \brief Handles a response from the server regarding getting a list of active participants.
+   *
+   * @param reply The response from the server.
+   */
+  void handleParticipantsResponse(QNetworkReply *reply);
 
   /**
    * \brief Prepares a network request that is going to include JSON.
@@ -584,13 +646,6 @@ private:
    */
   QUrl getPlayerPasswordUrl() const;
 
-  /**
-   * \brief Gets the url used for accessing the player's name.
-   *
-   * \return The url used for accessing the player's name.
-   */
-  QUrl getPlayerNameUrl() const;
-
   /**
    * \brief Gets the url used for accessing the player's location.
    *
@@ -619,6 +674,15 @@ private:
    */
   QUrl getCurrentSongUrl() const;
 
+  /**
+   * \brief Get the url to be used for retrieving the list of current participants.
+   *
+   * @return The url to be used for getting the current participants.
+   */
+  QUrl getParticipantsUrl() const;
+
+
+
   /**
    * \brief Gets the url that should be used for creating a player on the server.
    *
@@ -642,16 +706,6 @@ private:
    */
   QUrl getVolumeUrl() const;
 
-  /**
-   * \brief Determines whether or not the given path is the same one used by the
-   * player creation url.
-   *
-   * \param path The path in question.
-   * \return True if the path is the same as the path used by the player creation url,
-   * false otherwise.
-   */
-  bool isPlayerCreateUrl(const QString& path) const;
-
   /**
    * \brief Determines if the given network reply is in response to a request to 
    * obtain the active playlist.
@@ -720,7 +774,7 @@ private:
    * @return The port number to be used for communicating with the server.
    */
   static const QString & getServerPortNumber(){
-    static const QString serverPortNumber = "4898";
+    static const QString serverPortNumber = "4897";
     return serverPortNumber;
   }
 
@@ -731,7 +785,7 @@ private:
    */
   static const QString& getServerUrlPath(){
     static const QString SERVER_URL_PATH= 
-      "https://udjplayer.com:" + getServerPortNumber() + "/udj/";
+      "https://udjplayer.com:" + getServerPortNumber() + "/udj/0_6/";
     return SERVER_URL_PATH;
   }
 
